// GLOBAL FOR WHOLE APP :(
var widgetAPI = new Common.API.Widget();
var tvKey = new Common.API.TVKeyValue();
var pluginAPI = new Common.API.Plugin();

var TVPApp = new Navigable();

TVPApp.onLoad = function() {
  widgetAPI.sendReadyEvent();
  this.initComponents();
  var self = this;
  Locale.setLanguages(['ru'], {scope: this,
    fn:function() {
      self.authorize();
    }
  });
};

// public
TVPApp.authorize = function() {
  var self = this;
  WidgetsFactory.getWaitingScreen().show({message: Locale.t('Waiting.Authorize'),
    callback: { scope: self, fn: function() {widgetAPI.sendReturnEvent();}}});
  Auth.authorize({authComplete: {scope: this, fn: this.onAuthComplete},
    codeReceived: {scope: this, fn: this.onTokenCodeReceived}});
};

// public callback
TVPApp.onUnload = function() {
  alert("UNLOAD");
  this.hideCurrent();
  AppFactory.getRequestsFactory().abort();
};

// private callback
TVPApp.onTokenCodeReceived = function(tokenCode) {
  this.hideCurrent();
  WidgetsFactory.getConfirmTokenCodeScreen().show({tokenCode: tokenCode});
};


// private callback
TVPApp.onAuthComplete = function(authResult, errorDescription) {
  this.hideCurrent();

  if (AuthResults.SUCCESS == authResult) {
    AppFactory.getRequestsFactory().logInfo(AppInfo.getUserId(), AppInfo.getLogInfo());
    var self = this;
    var continueAfterVersionCheck = function() {
      self.mainMenu = WidgetsFactory.getMainMenu();
      self.mainMenu.show();
      self.updateNavi();
      self.setFirst();
    }

    CheckVersion.check({continueCallback: {scope: self, fn: continueAfterVersionCheck},
      exitCallback: {scope: self, fn: function() {widgetAPI.sendReturnEvent();}}
    });
  } else {
    WidgetsFactory.getLoginErrorScreen().show({authResult: authResult, errorDescription: errorDescription});
  }
};

// private callback
TVPApp.onScreenShown = function(screen, focused) {
  this.currentScreen = screen;
  this.updateNavi(screen, focused);
  WidgetsFactory.getNaviHelp().updateNavi(this.currentScreen.getNaviButtons());
};

// private callback
TVPApp.onDialogShown = function(dialog) {
  this.currentDialog = dialog;
  this.updateNavi(dialog, true);
  WidgetsFactory.getNaviHelp().updateNavi(this.currentDialog.naviButtons);

  $(getElById('MainLayout')).attr('class', 'main-layout');
  addStyleClassToEl('main-layout__modal', getElById('MainLayout'));
};

// private callback
TVPApp.onDialogHide = function(dialog) {
  this.currentDialog = null;
  if (this.currentScreen) {
    this.onScreenShown(this.currentScreen, true);
  } else {
    this.updateNavi();
  }
  $(getElById('MainLayout')).attr('class', 'main-layout');
};

// private
TVPApp.onMenuSelectionChanged = function(selectedScreenType, selectedSubType) {
  if (this.currentScreen && this.currentScreen.hasSubType(selectedSubType)) {
    this.currentScreen.processSubType(selectedSubType);
  } else {
    WidgetsFactory.getScreen(selectedScreenType).show({focused: false, subType: selectedSubType});
  }
};

// private
TVPApp.updateNavi = function(screen, focused) {
  var navi = [];
  if (this.mainMenu && (!screen || 'screen' == screen.mode)) {
    navi.push(this.mainMenu);
  }
  if (screen) {
    navi.push(screen);
  }
  this.setupNaviWithLine(navi, 'v', true);
  if (screen && focused) {
    this.setCurEl(screen, {saveCur: true});
  } else {
    this.setCurEl(this.mainMenu, {saveCur: true});
  }
};

// private
TVPApp.setFirst = function() {
  // TODO: предполагается, что достаем из local storage последний выбранный пользователем пункт меню
  // но пока что ставим Channels
  var menuItem = 'Channels';
  var menuSubItem = 'Favourites';

  var self = this;
  ModelsFactory.getChannels().get({ callback: {scope: self,
    fn: function(content) {
      if (!content || 0 == content.filter(function(el) { return el.favourited; }).length) {
        menuSubItem = 'All';
      }
      self.mainMenu.setItem({item:menuItem, subItem: menuSubItem, enter: true});
    }
  }});
}

// private
TVPApp.hideCurrent = function() {
  if (this.currentScreen) {
    this.currentScreen.hide();
  }
  if (this.currentDialog) {
    this.currentDialog.hide();
  }
}

// private
TVPApp.keyHandler = function(keyEvent) {
  this.escapeKeyHandler(keyEvent);
  this.naviKeyHandler(keyEvent);
  if (this.currentDialog) {
    this.currentDialog.keyHandler(keyEvent);
  } else if (this.currentScreen) {
    this.currentScreen.keyHandler(keyEvent);
  }
};

//private
TVPApp.naviKeyHandler = function(keyEvent) {
  var naviDirection = null;
  switch(keyEvent.keyCode) {
    case tvKey.KEY_LEFT :
      naviDirection = Navi.LEFT;
      break;
    case tvKey.KEY_RIGHT :
      naviDirection = Navi.RIGHT;
      break;
    case tvKey.KEY_UP :
      naviDirection = Navi.UP;
      break;
    case tvKey.KEY_DOWN :
      naviDirection = Navi.DOWN;
      break;
    case tvKey.KEY_ENTER:
    case tvKey.KEY_PANEL_ENTER:
      this.curEl.enter();
      break;
    case tvKey.KEY_TOOLS:
      if (!this.currentDialog) {
        this.updateNavi(this.currentScreen);
      }
      break;
    default :
      break;
  }
  if (naviDirection) {
    this.moveWhile(naviDirection);
  }
};

// private
TVPApp.escapeKeyHandler = function(keyEvent) {
  switch(keyEvent.keyCode)
  {
    case tvKey.KEY_RETURN:
    case tvKey.KEY_PANEL_RETURN:
      widgetAPI.blockNavigation(keyEvent);
      if (this.currentDialog) {
        this.currentDialog.return();
      } else if (this.currentScreen && this.currentScreen.hasParent()) {
        this.currentScreen.return();
      } else {
        widgetAPI.sendReturnEvent();
      }
      break;
    case tvKey.KEY_EXIT:
      if (this.currentDialog) {
        widgetAPI.blockNavigation(keyEvent);
        this.currentDialog.return();
      }
      break;
    default :
      break;
  }
};

// private
TVPApp.addListeners = function() {
  var self = this;

  $(WidgetsFactory.getMainMenu()).bind(Events.MENU_SELECTION_CHANGED,
    function(event, selectedScreenType, selectedSubType) {
      self.onMenuSelectionChanged(selectedScreenType, selectedSubType);
  });

  $(AppFactory.getContentProxy()).bind(Events.CONTENT_LOADING_START,
    function(event, contentRequestId) {
      var self = this;
      this.contentRequestId = contentRequestId;
      this.waitingDialog = WidgetsFactory.getWaitingScreen();
      this.waitingDialog.show({message: Locale.t('Waiting.Loading'), callback: { scope: self,
        fn: function() {
          if (self.contentRequestId) {
            AppFactory.getContentProxy().abort(self.contentRequestId);
          }
        }
      }});
  });

  $(AppFactory.getContentProxy()).bind(Events.CONTENT_LOADING_COMPLETE,
    function(event, contentRequestId) {
      if (contentRequestId == this.contentRequestId) {
        this.waitingDialog && this.waitingDialog.hide();
        this.contentRequestId = null;
      }
  });

  $(AppFactory.getContentProxy()).bind(Events.CONTENT_LOADING_ERROR,
    function(event, errorType) {
      WidgetsFactory.getErrorScreen().show({errorType: errorType});
  });



  $(AppFactory.getPlayer()).bind(Events.PLAYER_SCREEN_MODE_CHANGED,
    function(event) {
      if (event.info ==  AppFactory.getPlayer().FS) {
        $(WidgetsFactory.getToggleFSLayout()).hide();
      } else {
        $(WidgetsFactory.getToggleFSLayout()).show();
      }
  });

  $(WidgetsFactory.getRemoconFocusContainer()).keydown(
    function(keyEvent){ self.keyHandler(keyEvent);
  });
};

// private
TVPApp.initComponents = function() {
  WidgetsFactory.setScreenShownCallback({scope: this, fn: this.onScreenShown});
  WidgetsFactory.setDialogShownCallback({scope: this, fn: this.onDialogShown});
  WidgetsFactory.setDialogHideCallback({scope: this, fn: this.onDialogHide});
  $(WidgetsFactory.getRemoconFocusContainer()).focus();
  this.addListeners();
};

