var Auth = {

  // public
  authorize: function(callbacks) {
    this.completeCallback = callbacks.authComplete;
    this.confirmCallback = callbacks.codeReceived;

    AppFactory.getRequestsFactory().abort();
    clearInterval(this.confirmTimerId);
//AppInfo.setToken(null);
    if (AppInfo.getToken()){
      this.verifyToken();
    } else {
      this.requestToken();
    }
  },

  // private
  requestToken: function() {
    var self = this;
    AppFactory.getRequestsFactory().getToken(AppInfo.getDeviceId(), AppInfo.getDeviceType(), self, function(success, result) {
      // response like {"status":"success","code":"DPWFB5","retry_interval":3,"retry_duration":900}
      if (result && result.code && result.retry_interval && result.retry_duration) {
        self.confirmCallback.fn.call(self.confirmCallback.scope, result.code);
        var startTime = (new Date()).valueOf();
        var retryDuration = result.retry_duration * 1000;
        self.confirmTimerId = setInterval(function() {
          self.checkTokenConfirmed(result.code, startTime, retryDuration)    
        }, result.retry_interval * 1000);
      } else {
        self.completeCallback.fn.call(self.completeCallback.scope, AuthResults.NETWORK_ERROR, result);
      }
    });
  },

  // private
  verifyToken: function() {
    var self = this;
    AppFactory.getRequestsFactory().verifyToken(this, function(success, result) {
      var r = this.parseAuthResult(result);
      if (r) {
        if (AuthResults.UNAUTHORIZED == r) {
          self.requestToken();// TODO: clear token ?
        } else {
          self.completeCallback.fn.call(self.completeCallback.scope, r);
        }
      } else  {
        self.completeCallback.fn.call(self.completeCallback.scope, AuthResults.NETWORK_ERROR, result);
      }
    });
  },

  // private
  checkTokenConfirmed: function(code, startTime, retryDuration) {
    var self = this;
    AppFactory.getRequestsFactory().checkTokenConfirmed(AppInfo.getDeviceId(), AppInfo.getDeviceType(), code, self, function(success, result) {
      if (result && result.token) {
        AppInfo.setToken(result.token);
        AppInfo.setUserId(result.user_id);
      }
      var elapsedTime = ((new Date()).valueOf() - startTime);
      var r = this.parseAuthResult(result);
      if (AuthResults.SUCCESS  == r || AuthResults.STB_INACTIVE == r || AuthResults.SUBSCRIPTION_INACTIVE == r) {
        clearInterval(self.confirmTimerId);
        self.completeCallback.fn.call(self.completeCallback.scope, r);
      } else if ((retryDuration <= elapsedTime)) {
        clearInterval(self.confirmTimerId);
        self.completeCallback.fn.call(self.completeCallback.scope, AuthResults.CONFIRM_TOKEN_FAIL);
      }
    });
  },

  // private
  parseAuthResult: function(r) {
    var result = null;
    if (r) {
      if (r.hasOwnProperty('stb_active') && !r.stb_active)
        result = AuthResults.STB_INACTIVE;
      if (r.hasOwnProperty('subscription_active') && !r.subscription_active)
        result = AuthResults.SUBSCRIPTION_INACTIVE;
      if (r.hasOwnProperty('authorized') && !r.authorized)
        result = AuthResults.UNAUTHORIZED;
      if (r.authorized && r.stb_active && r.subscription_active)
        result = AuthResults.SUCCESS;
    }
    return result;
  }
}
