function Channels(proxy) {
  this.proxy = proxy;
  this.channels = [];
}

// public
// config {
//   update: boolean, get from server forced (default false)
//   sync: boolean (default false), N.B.: it locks the app
//   callback: {scope, fn}
//   start: int, start index (default 0)
//   count: int
//}
Channels.prototype.get = function(config) {
  var c = $.extend({update: false, sync: false, start:0}, config);

  if (c.update) {
    this.channels = [];
    this.totalCount = null;
    c.update = false;
  }

  if (0 < this.channels.length || c.prevLoadFailed) {
    this.getChannelsResult = this.getChannels(c.start, c.count);
    c.callback && c.callback.fn.call(c.scope, this.getChannelsResult);
  } else {
    var self = this;
    this.proxy.getChannels({scope: self,
      fn: function(success, result) {
        self.onChannelsLoaded(success, result);
        c.prevLoadFailed = !(success && result);
        this.getChannelsResult = self.get(c);
      }
    }, {sync: c.sync});
  }

  return this.getChannelsResult;
}

// public
Channels.prototype.totalCount = function() {
  return this.totalCount;
}

// public
// config {
//   channel,
//   callback: {scope, fn}
// }
Channels.prototype.toggleFavourited = function(config) {
  var c = config;
  var self = this;
  this.proxy.toggleChannelFavourited(c.channel, { scope: self,
    fn: function(success, result) {
      var successResult = result && result.hasOwnProperty('abbr')
      if (successResult) {
        for(var i = 0; i < self.channels.length; i++) {
          if (result.abbr == self.channels[i].abbr) {
            self.channels[i].favourited = !self.channels[i].favourited;//result.favourited; //TODO: get real fav state
          }
        }
      }
      c.callback && c.callback.fn.call(c.callback.scope, successResult, this.channels);
    }
  });
}


// private callback
Channels.prototype.onChannelsLoaded = function(success, result) {
  if (success && result) {
    if (result.channels) {
      this.channels = result.channels;
      this.totalCount = this.channels.length;
      for (var i = 0; i < this.channels.length; i++) {
        this.channels[i].liveUrls = this.channels[i][AppInfo.streamUrlsPrefix()+"_live_urls"];
        this.channels[i].timeshiftUrls = this.channels[i][AppInfo.streamUrlsPrefix()+"_timeshift_urls"];
      }
    }
  }
}

// private
Channels.prototype.getChannels = function(start, count) {
  var c = null;
  if (start < this.totalCount) {
    c = [];
    var endIndex = Math.min(start + (count || this.totalCount), this.totalCount);
    for (var i = start; i < endIndex; i++) {
      c.push(this.channels[i]);
    }
  }
  return c;
}

