function ContentLoadProxy(requestsFactory) {
  this.requests = requestsFactory;

  this.requestsMap = {};
  this.requestId = 0;
}

// public
ContentLoadProxy.prototype.abort = function(requestId) {
  this.requests.abort(requestId);
  this.onLoadComplete(requestId);
}

// public
ContentLoadProxy.prototype.getQueue = function(page, callbacks, options) {
  this.makeRequest(callbacks, options, this.requests.getQueue, page);
}

// public
ContentLoadProxy.prototype.removeFilmFromQueue = function(queuedId, callbacks, options) {
  this.makeRequest(callbacks, options, this.requests.removeFilmFromQueue, queuedId);
}

// public
ContentLoadProxy.prototype.getChannels = function(callbacks, options) {
  this.makeRequest(callbacks, options, this.requests.getChannels);
}

// public
ContentLoadProxy.prototype.toggleChannelFavourited = function(channel, callbacks, options) {
  this.makeRequest(callbacks, options,
    channel.favourited ? this.requests.removeChannelFromFav : this.requests.addChannelToFav, channel.abbr);
}

// public
ContentLoadProxy.prototype.getTelecasts = function(channelAbbr, callbacks, options) {
  this.makeRequest(callbacks, options, this.requests.getChannelTelecasts, channelAbbr);
}

// public
ContentLoadProxy.prototype.getAccount = function(callbacks, options) {
  this.makeRequest(callbacks, options, this.requests.getAccount);
}

// public
ContentLoadProxy.prototype.getLastVersion = function(callbacks, options) {
  this.makeRequest(callbacks, options, this.requests.getLastVersion, Settings.lastVersion.domain);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////

// private
ContentLoadProxy.prototype.addCallbacks = function(callbacks) {
  var self = this;
  return [].concat(callbacks, {scope: self, sfn: self.onLoadComplete, ffn: self.onLoadFail});
}

// private
ContentLoadProxy.prototype.makeRequest = function(callbacks, options, requestAction, requestArgs) {
  this.requestId++;
  $(this).trigger(Events.CONTENT_LOADING_START, this.getRequestId());
  var args = [].concat((requestArgs || []),[this.addCallbacks(callbacks)], options, this.getRequestId());
  requestAction.apply(this.requests, args);
}

// private
ContentLoadProxy.prototype.onLoadComplete = function(requestId) {
  alert("ContentLoadProxy.onLoadComplete() " + requestId);

  $(this).trigger(Events.CONTENT_LOADING_COMPLETE, requestId);
  var r = this.requestsMap[requestId];
  delete this.requestsMap[requestId];
}

// private
ContentLoadProxy.prototype.onLoadFail = function(requestId, error) {
  alert("ContentLoadProxy.onLoadFail() " + error);

  this.onLoadComplete(requestId);
  $(this).trigger(Events.CONTENT_LOADING_ERROR, error);
}

// private
ContentLoadProxy.prototype.getRequestId = function() {
  return this.requestId + 'c';
}


