function MainMenu(rootEl) {
  this.rootEl = rootEl;
};

MainMenu.inheritsFrom(Navigable);

// public
MainMenu.prototype.show = function() {
  $(this.rootEl).show();

  var html = new EJS({url: 'templates/mainMenu.ejs'}).render();
  widgetAPI.putInnerHTML(this.rootEl, html);

  this.itemsNavi = WidgetsFactory.mainMenu.getItemsNavi();

  var self = this;
  $(this.itemsNavi).bind(Events.NAVI_SELECTION_SET,
    function(event, selectedItem, changed) {
      self.onSelectionChanged(selectedItem, changed);
  });

  this.setupNavi([[this.itemsNavi]]);
}

/**
 * config {
 *   item
 *   subItem
 *   focus (item or subitem)
 *   enter (bool)
 * }
 **/
//public
MainMenu.prototype.setItem = function(config) {
  if (!config) {
    return;
  }
  this.updateSubItems(config.item);

  var itemRootEl = WidgetsFactory.mainMenu.getItemRootEl(config.item);
  var subItemRootEl = WidgetsFactory.mainMenu.getSubItemRootEl(config.item, config.subItem);

  if (config.subItem && config.focus == config.subItem) {
    this.setCurEl(this.subItemsNavi);
    this.subItemsNavi.setCurEl(subItemRootEl);
  } else {
    this.setCurEl(this.itemsNavi);
    this.itemsNavi.setCurEl(itemRootEl);
  }

  if (!config.enter) {
    this.itemsNavi.setSelected(itemRootEl);
    if (config.subItem && this.subItemsNavi) {
      this.subItemsNavi.setSelected(subItemRootEl);
    }
  } else {
    if (config.subItem && this.subItemsNavi) {
      this.itemsNavi.setSelected(itemRootEl);
      this.subItemsNavi.setCurEl(subItemRootEl, {withoutFocus: (config.focus != config.subItem)});
      this.subItemsNavi.enter();
    } else {
      this.enter();
    }
  }
}


// private
MainMenu.prototype.onSelectionChanged = function(selectedItem, changed) {
  if (!changed) {
    return;
  }
  this.updateSubItems(getElLastName(selectedItem));
  if (this.subItemsNavi) {
    this.subItemsNavi.setCurEl(null, {withoutFocus: true});
    this.subItemsNavi.enter();
  } else {
    this.setupNavi([[this.itemsNavi]]);
    $(this).trigger(Events.MENU_SELECTION_CHANGED, [getElLastName(selectedItem)]);
  }
}

// private
MainMenu.prototype.updateSubItems = function(item) {
  this.clearCurrentSubItems();
  this.subItemsNavi = WidgetsFactory.mainMenu.getSubItemsNavi(item);
  if (this.subItemsNavi) {
    var self = this;
    $(this.subItemsNavi).bind(Events.NAVI_SELECTION_SET,
      function(e, selectedItem){
        self.onSubSelectionChanged(selectedItem);
    });

    this.subItemsRoot = WidgetsFactory.mainMenu.getSubItemsRootEl(item);
    $(this.subItemsRoot).show();

    this.setupNaviWithLine([this.itemsNavi, this.subItemsNavi], 'v');
  }
}

// private
MainMenu.prototype.clearCurrentSubItems = function() {
  if (this.subItemsRoot) {
    $(this.subItemsRoot).hide();
  }
  if (this.subItemsNavi) {
    this.subItemsNavi.reset();
  }
}

MainMenu.prototype.onSubSelectionChanged = function(selectedItem) {
  // TODO: replace to Utils
  if (selectedItem && selectedItem.id) {
    var m = selectedItem.id.match(/((\w+)\.)*(\w+)/i);
    if (m && 2 <= m.length) {
      $(this).trigger(Events.MENU_SELECTION_CHANGED, [m[m.length - 2], m[m.length - 1]]);
    }
  }
}

