/**
 * config: {
 *   pageSize
 *   rowsCount
 *   source
 * }
 */
function PagedDisplay(rootEl, config) {
  this.rootEl = rootEl;

  this.updateConfig(config);
}

PagedDisplay.inheritsFrom(Navigable);

// public
PagedDisplay.prototype.updateConfig = function(config) {
  $.extend(this, config);
}

/**
 * d - direction (ability to navigate from back)
 * fp - position (ability to fix row or column on page changed) 
 */
// public
PagedDisplay.prototype.showPage = function(page, d, fp) {
  $(this.rootEl).show();
  var items = this.getItems((page || 0) * this.pageSize, this.pageSize);
  if (0 == items.length && 0 < page) {
    return false;
  }
  this.curPage = page || 0;
  this.setupNavi(this.renderPage(items), {vShift: this.vShift, hShift: this.hShift});
  d && this.setCurEl(null, {dir:d, fixed: fp});
  return this.navi.curEl();
}

// public
PagedDisplay.prototype.showPageWithEl = function(findEl, focused) {
  var index = -1;
  for (var i = 0; i < this.source.length; i++) {
    if (findEl.fn.call(findEl.scope, this.source[i])) {
      index = i;
      break;
    }
  }
  if (index != -1) {
    var page = Math.floor(index/this.pageSize);
    this.showPage(page);
    if (focused) {
      var pageOffset = index - page * this.pageSize;
      this.setCurEl(null, {pos: this.getPosByPageOffset(pageOffset)});
    }
  }
}

// public
PagedDisplay.prototype.hide = function() {
  $(this.rootEl).hide();
}

// public override
PagedDisplay.prototype.move = function(d) {
  var cp = this.navi.currentPos();
  if( d.dy == 1 && (cp.y == this.rowsCount - 1)) {
    return this.showPage(this.curPage + 1, d, cp);
  }
  if ( d.dy == -1 && (cp.y == 0) && this.curPage != 0) {
    return this.showPage(this.curPage - 1, d, cp);
  }

  // don't use parent because it makes recursion on inheritance chain
  return Navigable.prototype.move.call(this, d);
}

// protected
PagedDisplay.prototype.getItems = function(startIndex, count) {
  return this.source ? this.source.slice(startIndex, startIndex + count) : [];
}

// protected
PagedDisplay.prototype.renderPage = function(items) {
  return [[]]; // navi table
}

// protected for override
PagedDisplay.prototype.getPosByPageOffset = function(pageOffset) {
  return {x: 0, y: 0}
}

