function Navigable() {
  this.hoveredStyle = 'focused';
  this.selectedStyle = 'selected';
  this.selectionSetEvent = Events.NAVI_SELECTION_SET;

  this.changeCursorReasons = [Navi.UPDATING, Navi.MOVING, Navi.RESET];
  this.setCursorReasons = [Navi.FOCUSING];
}

// public
Navigable.prototype.setupNavi = function(naviTable, border, pos) {
  this.navi = this.navi ? this.navi.update(this, naviTable, border, pos) 
  : new Navi(this, naviTable, border, pos);
}

// public
Navigable.prototype.setupNaviWithLine = function(items, direction, shift, pos) {
  var table;
  var border;
  if ('v' == direction) {
    table = [[]];
    for(var i = 0; i < items.length; i++) {
      table[0].push(items[i]);
    }
    border = {top: 0, bottom: items.length - 1, left: 0, right: 0, vShift: shift}
  } else if ('h' == direction) {
    table = [];
    for(var i = 0; i < items.length; i++) {
      table[i] = [items[i]];
    }
    border = {top: 0, bottom: 0, left: 0, right: items.length - 1, hShift: shift};
  }
  this.setupNavi(table, border, pos);
}

// public
Navigable.prototype.move = function(d) {
  //alert("Navigable.move() " + this.id + " " + d.dx + " " + d.dy + " ; curEl " + (this.curEl && this.curEl.id));
  if (!(this.curEl && this.curEl.move && this.curEl.move(d))) {
    var moveResult = this.navi && this.navi.move(d);
    return !this.isNaviEmpty() && moveResult;
  }
  return this.curEl;
}

// public
/**
 * ignore empty navi elements, move by direction until filled navi element is found
 */
// TODO: протестировать
Navigable.prototype.moveWhile = function(d) {
  var moved = false;
  var steps = 0;
  var maxSteps = (d.dx != 0) ? this.navi.naviTable.length : this.navi.naviTable[this.navi.currentPos().x].length;
  while ((!moved || this.isNaviEmpty()) && steps <= maxSteps) {
    moved = this.move(d);
    steps++;
  }

  return this.curEl;
}

// public
Navigable.prototype.enter = function() {
  if (this.curEl && this.curEl.enter) {
    this.curEl.enter();
  } else {
    var changed = (this.hoveredItem != this.selectedItem);
    this.selectItem(this.hoveredItem);
    $(this).trigger(this.selectionSetEvent, [this.getSelection(), changed]);
  }
}

// public
Navigable.prototype.setCurEl = function(el, conf) {
  //alert("@@@ Navigable.setCurEl() " + (this.rootEl && this.rootEl.id));
  this.navi && this.navi.setCurEl(el, conf);
}

// public
Navigable.prototype.setSelected = function(el) {
  this.selectItem(el);
}

// public
Navigable.prototype.reset = function() {
  //alert("@@@ Navigable.reset() " + this.id );
  if (this.navi) {
    var items = this.navi.getAllItems();
    for (var i = 0; i < items.length; i++) {
      items[i] && items[i].reset && items[i].reset();
    }
    this.removeFocus();
    this.resetSelected();
    this.navi.reset();
  }
}

// TODO: переименовать, причесать
// public
Navigable.prototype.isNaviEmpty = function() {
  var isEmpty = !this.navi || this.navi.isEmpty() || (this.curEl && this.curEl.isNaviEmpty && this.curEl.isNaviEmpty());
  return isEmpty;
}

////////////////////////////////////////////////////////////////////////

// protected
Navigable.prototype.hoverItem = function(item, withoutFocus) {
  //alert("Navigable.hoverItem() " + this.id + " " + (item && item.id));
  if (this.hoveredItem){
    removeStyleClassFromEl(this.hoveredStyle,  this.hoveredItem);
  }
  this.hoveredItem = item;

  if (this.hoveredItem && !withoutFocus) {
    addStyleClassToEl(this.hoveredStyle,  this.hoveredItem);
  }
}

// protected
Navigable.prototype.selectItem = function(item) {
  //alert("Navigable.selectItem() ");
  if (this.selectedItem) {
    removeStyleClassFromEl(this.selectedStyle,  this.selectedItem);
  }
  this.selectedItem = item;
  if (this.selectedItem) {
    addStyleClassToEl(this.selectedStyle,  this.selectedItem);
  }
}

// protected
Navigable.prototype.resetSelected = function() {
  //alert("Navigable.resetSelected() " + this.id );
  if (this.selectedItem) {
    removeStyleClassFromEl(this.selectedStyle,  this.selectedItem);
    this.selectedItem.resetSelected && this.selectedItem.resetSelected();
    this.selectedItem = null;
  }
}

// protected
Navigable.prototype.getSelection = function() {
  return this.selectedItem;
}

///////////////////////////////////////////////////////////

// private
Navigable.prototype.updateFocus = function(el, conf) {
  //alert("@@@ Navigable.updateFocus() " + (this.rootEl && this.rootEl.id));
  this.removeFocus();
  this.curEl = el;
  this.hoverItem(this.curEl, (conf && conf.withoutFocus));
  if(this.curEl && this.curEl.setCurEl) {
    this.curEl.setCurEl(null, conf);
  }
}

// private
Navigable.prototype.removeFocus = function() {
  //alert("Navigable.removeFocus() " + this.id + " " + (this.hoveredItem && this.hoveredItem.id));
  if (this.hoveredItem) {
    // sometimes it throws strange jQury error for smarttv 2011
    try {
      removeStyleClassFromEl(this.hoveredStyle,  this.hoveredItem);
      this.hoveredItem.removeFocus && this.hoveredItem.removeFocus();
    }
    catch(e) {
      alert("Navigable.removeFocus() error ");
    }
    this.hoveredItem = null;
  }
}

// callback for Navi
Navigable.prototype.onCursorSet = function(reason, conf) {
  //alert("Navigable.onCursorSet() " + this.id + " " + reason);
  arrContains(this.setCursorReasons, reason) && this.updateFocus(this.navi.curEl(), conf);
}

// callback for Navi
Navigable.prototype.onCursorChanged = function(reason, conf) {
  arrContains(this.changeCursorReasons, reason) && this.updateFocus(this.navi.curEl(), conf);
}
